
/*									*/
/*    Lattice C interface to the Btrieve Record Manager, version 4	*/
/*									*/
/*	Note: if compiling for the "D" or "L" memory model, remove      */
/*	      the comments surrounding the following definition.	*/
/*									*/
/*				    LMODEL means 32-bit pointers in use */
/* #define LMODEL 1	 */


#define BTR_ERR     20			  /* record manager not started */
#define BTR_INT     0x7B		    /* Btrieve interrupt vector */
#define BTR2_INT    0x2F		 /* multi-user interrupt vector */
#define BTR_VECTOR  BTR_INT * 4 		/* offset for interrupt */
#define BTR_OFFSET  0x33	       /* Btrieve offset within segment */
#define VARIABLE_ID 0x6176     /* id for variable length records - 'va' */
#define _2FCODE     0xAB00	 /* function code for int 2F to btrieve */

/* ProcId is used for communicating with the Multi Tasking Version of  */
/* Btrieve. It contains the process id returned from BMulti and should */
/* not be changed once it has been set. 			       */
/*								       */

static unsigned ProcId = 0;		/* initialize to no process id */
static char MULTI = 0;		      /* flag set to true if MultiUser */
static char VSet = 0;		/* flag set to true if checked version */

BTRV (OP, POS_BLK, DATA_BUF, DATA_LEN, KEY_BUF, KEY_NUM)
  int  OP;
  char POS_BLK[];
  char DATA_BUF[];
  int  *DATA_LEN;
  char KEY_BUF[];
  int  KEY_NUM;

{
struct REGVAL { int AX, BX, CX, DX, SI, DI; } REGS;

struct SEGREG { short ES, CS, SS, DS; } SREGS;

struct BTRIEVE_PARMS	  /* structure passed to Btrieve Record Manager */
 {
   char *BUF_OFFSET;			  /* callers data buffer offset */
#ifndef LMODEL
   int	 BUF_SEG;			 /* callers data buffer segment */
#endif
   int	 BUF_LEN;			       /* length of data buffer */
   char *CUR_OFFSET;			  /* user position block offset */
#ifndef LMODEL
   int	 CUR_SEG;			 /* user position block segment */
#endif
   char *FCB_OFFSET;				  /* offset of disk FCB */
#ifndef LMODEL
   int	 FCB_SEG;				 /* segment of disk FCB */
#endif
   int	 FUNCTION;				  /* requested function */
   char *KEY_OFFSET;			 /* offset of user's key buffer */
#ifndef LMODEL
   int	 KEY_SEG;			/* segment of user's key buffer */
#endif
   char  KEY_LENGTH;			 /* length of user's key buffer */
   char  KEY_NUMBER;			/* key of reference for request */
   int	*STAT_OFFSET;			       /* offset of status word */
#ifndef LMODEL
   int	 STAT_SEG;			      /* segment of status word */
#endif
   int	 XFACE_ID;				 /* language identifier */
 } XDATA;

int STAT = 0;				      /* status of Btrieve call */

/*									*/
/*  Check to see that the Btrieve Record Manager has been started.	*/
/*									*/

REGS.AX = 0x3500 + BTR_INT;
int86x (0x21, &REGS, &REGS, &SREGS);
if (REGS.BX != BTR_OFFSET)
   return (BTR_ERR);

if (!VSet)
 {
  VSet = 1;
  REGS.AX = 0x3000;
  int86x (0x21, &REGS, &REGS, &SREGS);
  if ((REGS.AX & 0x00FF) >= 3)
   {
    REGS.AX = _2FCODE;
    int86x (BTR2_INT, &REGS, &REGS, &SREGS);
    MULTI = ((REGS.AX & 0xFF) == 'M');
   }
 }

/*  Read segment registers and initialize segment part of addresses to	*/
/*  user's data segment.                                                */
/*									*/

segread (&SREGS);
#ifndef LMODEL
XDATA.BUF_SEG = XDATA.CUR_SEG = XDATA.FCB_SEG =
  XDATA.KEY_SEG = XDATA.STAT_SEG = SREGS.SS;
#endif

/*									*/
/*  Move user parameters to XDATA, the block where Btrieve expects them.*/
/*									*/

XDATA.FUNCTION	  = OP;
XDATA.STAT_OFFSET = &STAT;
XDATA.FCB_OFFSET  = POS_BLK;
XDATA.CUR_OFFSET  = POS_BLK + 38;
XDATA.BUF_OFFSET  = DATA_BUF;
XDATA.BUF_LEN	  = *DATA_LEN;
XDATA.KEY_OFFSET  = KEY_BUF;
XDATA.KEY_LENGTH  = 255;		 /* use max since we don't know */
XDATA.KEY_NUMBER  = KEY_NUM;
XDATA.XFACE_ID	  = VARIABLE_ID;

/*									*/
/*  Make call to the Btrieve Record Manager.				*/
/*									*/

#ifdef LMODEL
REGS.DX = (int) FP_OFF(&XDATA); 	       /* parameter block in DX */
#else
REGS.DX = (int) &XDATA;
#endif

SREGS.DS = SREGS.SS;
if (!MULTI)
  int86x (BTR_INT, &REGS, &REGS, &SREGS);
else
 {
  while (1)
   {
    REGS.AX = 1;
    if ((REGS.BX = ProcId) != 0)
      REGS.AX = 2;
    REGS.AX += _2FCODE;
#ifdef LMODEL
    REGS.DX = (int) FP_OFF(&XDATA);	      /* parameter block in DX */
#else
    REGS.DX = (int) &XDATA;
#endif
    SREGS.DS = SREGS.SS;
    int86x (BTR2_INT, &REGS, &REGS, &SREGS);
    if ((REGS.AX & 0x00FF) == 0) break;
    REGS.AX = 0x0200;
    int86x (0x7F, &REGS, &REGS, &SREGS);
   }
  if (ProcId == 0) ProcId = REGS.BX;
 }

*DATA_LEN = XDATA.BUF_LEN;
return (STAT);					       /* return status */
}
