REM-----------------------------------------------------------------------
REM
REM This routine interfaces between Borland's Turbo Basic and
REM Btrieve on PC-DOS machines.  Turbo Basic is compiled.
REM
REM To use, use the metastatement $include:
REM             $INCLUDE "TURBBTRV.BAS"
REM
REM Calling Procedure from Turbo Basic
REM   CALL BTRV (OP%, STATUS%, POSITION$, FILEBUF$, DATALEN%,
REM                     KEYBUF$, KEYNUM%)   where
REM     OP% - function number for request
REM     STATUS% - return status from request
REM     POSITION$ - 128 byte data area for FCB and positioning,
REM       not to be changed by user
REM     FILEBUF$ - address of data buffer from FIELDING statement
REM     DATALEN% - length of user data buffer
REM     KEYBUF$ - string pointer to key buffer
REM     KEYNUM% - key number to be processed
REM
REM Important notes:
REM  This interface changes the data segment which is used by BLOAD, BSAVE,
REM    CALL ABSOLUTE, PEEK, and POKE, because it does a DEF SEG.  Thus, if
REM    these commands are used after a call to BTRV, it is important to do
REM    a DEF SEG after the BTRV call and before using the above statements.
REM
REM  Unlike some BASIC interpreters, Turbo Basic checks that the number and
REM   type of the arguments to a procedure (such as BTRV) agree with the
REM   formal parameters.
REM
REM  Static variables are not visible outside the subroutine, so it is
REM    not possible for another part of the application to change them.
REM    They must be static because their values must be retained between
REM    calls to BTRV.
REM
REM  Note that all string parameters must be within the string segment.
REM  Varptr cannot be used to get the address of the string parameters.
REM  It returns a pointer to the string descriptor, of which the first two
REM  bytes are the length and the next two bytes are the offset of the
REM  string WITHIN the default string segment.  The value of the default
REM  string segment is stored in the default segment:0.  The default segment
REM  can be obtained by doing a DEF SEG with no arguments.
REM
REM History:
REM     5/22/87- New for Turbo Basic.
REM
REM-----------------------------------------------------------------------


SUB BTRV (OP%, STATUS%, POSITION$, FILEB$, DATALEN%, KEYBUF$, KEYNUM%)

STATIC VSET%            : REM these variables must be static because
STATIC MULTI%           : REM   values must be retained between calls
STATIC PROCID%          : REM   to BTRV
LOCAL PARMBLOCK%        : REM local parameter block
DIM A% (14)             : REM dimension param block as integer array
                        : REM save error vector variables
LOCAL ERRVECO1A%, ERRVECO1B%, ERRVECO2A%, ERRVECO2B%
LOCAL STRINGSEG%        : REM string segment
LOCAL TEMP%             : REM temporary for byte to word conversions
                        : REM define indices within parm block
%databufo = 0   : %databufs = 1
%dblen    = 2
%posoff   = 3   : %posseg   = 4
%fcboff   = 5   : %fcbseg   = 6
%bop      = 7
%keybufo  = 8   : %keybufs  = 9         : %keynum   = 10
%statoff  = 11  : %statseg  = 12
%varid    = 13
%poslen   = 128 : %poslenerr = 23       : %btrloderr = 20
%axreg = 1      : %bxreg = 2            : %dxreg = 4
%dsreg = 8

    DEF SEG  = 0                : REM  must save basic's disk error
    ERRVECO1A% = PEEK (&H90)
    ERRVECO1B% = PEEK (&H91)
    ERRVECO2A% = PEEK (&H92)
    ERRVECO2B% = PEEK (&H93)
    POKE &H90, PEEK (&H51A)
    POKE &H91, PEEK (&H51B)     : REM   and set DOS to handle fatal
    POKE &H92, PEEK (&H51C)     : REM   errors
    POKE &H93, PEEK (&H51D)
                                : REM ensure Btrieve has been loaded
    REG %axreg, &H357B          : REM   by using fct 35, get vector
    CALL INTERRUPT &H21         : REM   with Btrieve's interrupt
                                : REM if Btrieve's code offset, ok
    IF REG (%bxreg) = &H33, THEN
      IF VSET% = 0, THEN
        INCR VSET%              : REM if version not set, set it
        REG %axreg, &H3000      : REM  get DOS version number

        CALL INTERRUPT &H21
        IF (REG (%axreg) AND &H00FF) < 3, THEN
        ELSE                    : REM if major version >= 3
          REG %axreg, &HAB00
          CALL INTERRUPT &H2F   : REM check for bmulti being loaded
          IF ((REG (%axreg) AND &H00FF) = 77), THEN
            LET MULTI% = 1      : REM yes, bmulti
          ELSE
            LET MULTI% = 0
          END IF                : REM set bmulti flag
        END IF                  : REM bmulti possible only if DOS above 3.x
      END IF                    : REM set ver of Btrieve loaded (multi or not)

      DEF SEG
      CALL SETSSEG (STRINGSEG%)

      LET A%(%bop)     = OP%                    : REM set function number
      LET A%(%statoff) = VARPTR (STATUS%)       : REM status addr
      LET A%(%statseg) = VARSEG (STATUS%)
      IF (LEN (POSITION$) < %poslen), THEN
         STATUS% = %poslenerr                   : REM check pos block len
         EXIT SUB
      END IF

      DEF SEG = VARSEG (POSITION$)              : REM fcb addr
      CALL SETWORD (A%(%fcboff), POSITION$)
      LET A%(%fcbseg) = STRINGSEG%
                                                : REM position block
      LET A%(%posoff)  = A%(%fcboff) + 38
      LET A%(%posseg)  = STRINGSEG%

      DEF SEG = VARSEG (FILEB$)                 : REM data buffer addr
      CALL SETWORD (A%(%databufo), FILEB$)
      LET A%(%databufs) = STRINGSEG%

      LET A%(%dblen)  = DATALEN%                : REM data buf len

      DEF SEG = VARSEG (KEYBUF$)                : REM key buffer address
      CALL SETWORD (A%(%keybufo), KEYBUF$)
      LET A%(%keybufs) = STRINGSEG%

      CALL SETKEY (A%(%keynum), KEYNUM%, KEYBUF$)       :REM key len, number

      LET A%(%varid) = &H6176                   : REM variable id

      REG %dsreg, VARSEG (A%(0))        : REM set up ds:dx = data segment
      REG %dxreg, VARPTR (A%(0))        : REM   and offset => param block


      IF (MULTI% = 0), THEN
        CALL INTERRUPT &H7B             : REM call Btrieve
      ELSE                              : REM bmulti loaded
        DO                              : REM make request
          IF PROCID% = 0, THEN
             REG %axreg, &HAB01         : REM set function to get proc id
          ELSE
             REG %bxreg, PROCID%
             REG %axreg, &HAB02         : REM set function, have proc id
          END IF
          CALL INTERRUPT &H2F           : REM call bmulti
          IF ( ((REG (%axreg) AND &H00FF) = 0) ), THEN
             EXIT LOOP                  : REM request processed by bmulti
          END IF
          REG %axreg, &H200             : REM not processed,
          CALL INTERRUPT &H7F           : REM  allow task switch
        LOOP                            : REM and make request again

        IF (PROCID% = 0), THEN
          LET PROCID% = REG (%bxreg)    : REM set procid
        END IF                          : REM didn't have before

      END IF
      DATALEN% = A%(2)                  : REM set up new db len
    ELSE
      STATUS% = %btrloderr              : REM Btrieve not loaded
    END IF

    DEF SEG = 0                         : REM restore error vector
    POKE &H90,ERRVECO1A%
    POKE &H91,ERRVECO1B%
    POKE &H92,ERRVECO2A%
    POKE &H93,ERRVECO2B%
END SUB

SUB SETSSEG INLINE              'set string segment from default ds:0 to parm
                                '  current ds must be tbasic default ds
                                ' setsseg (integer to hold stringseg)
  $INLINE &H55                  'push bp
  $INLINE &H89, &HE5            'mov bp,sp
  $INLINE &HC4, &H7E, &H06      'les di,[bp+06]
  $INLINE &HBB, &H00, &H00      'mov bx,0
  $INLINE &H8B, &H07            'mov ax,[bx]
  $INLINE &H26, &H89, &H05      'es: mov [di],ax
  $INLINE &H5D                  'pop bp
END SUB

SUB SETWORD INLINE              'set array entry with offset in parm
                                ' set word (array entry, variable addr)
  $INLINE &H55                  'push bp
  $INLINE &H89, &HE5            'mov bp,sp
  $INLINE &H8B, &H5E, &H06      'mov bx,[bp+06]   offset of source
  $INLINE &H8B, &H46, &H08      'mov ax,[bp+08]   segment of source
  $INLINE &H8E, &HC0            'mov es,ax
  $INLINE &H26                  'es:
  $INLINE &H8B, &H47, &H02      'mov ax,[bx+2]
  $INLINE &HC4, &H7E, &H0A      'les di,[bp+0A]   array element
  $INLINE &H26, &H89, &H05      'es: mov [di],ax
  $INLINE &H5D                  'pop bp
END SUB

SUB SETKEY INLINE               'set key length/number array element
                                ' setkey (array entry, keynum, keybuf)
  $INLINE &H55                  'push bp
  $INLINE &H89, &HE5            'mov bp,sp
  $INLINE &HC4, &H7E, &H0A      'les di,[bp+0a]  key number pointer
  $INLINE &H26, &H8B, &H0D      'mov cx,es:[di]  key number value
  $INLINE &HC4, &H7E, &H06      'les di,[bp+06]  off,seg of key buf
  $INLINE &H26, &H8B, &H05      'mov ax,es:[di]  key length
  $INLINE &H88, &HCC            'mov ah,cl
  $INLINE &HC4, &H7E, &H0E      'les di,[bp+0e]  off,seg of array entry
  $INLINE &H26, &H89, &H05      'mov es:[di],ax
  $INLINE &H5D                  'pop bp
END SUB

